--[[
File: Roles.lua
Author: Zormus
--]]

local Utility = Library.LibZormus.Utility
local addon, Roles = ...
Roles.roles = {}
local roles = Roles.roles
local menu = Roles.menu
local config = Roles.config

local player, currentRole

local function UpdateMenu(update)
	if menu:IsOpen() or update then
		table.sort(roles, Roles.Compare[_config.sort])
		local count = 0
		for _, role in ipairs(roles) do
			if role:GetMenu() then
				count = count + 1
				menu:UpdateItem(count, role)
			end
		end
		menu:SetCount(count)
		table.sort(roles, Roles.Compare.Slot)
	end
end

function menu:OnOpen()
	Roles.UpdateList(roles)
	self:UpdateButton(currentRole)
	UpdateMenu(true)
end

local function ShowConfig()
	if not config:IsOpen() then
		menu:SetActive(false)
		Roles.UpdateList(roles)
		menu:UpdateButton(currentRole)
		config:Show()
	end
end

function config:OnClose()
	for _, role in ipairs(roles) do
		role:UpdateMacros()
	end
	if not Inspect.System.Secure() then
		menu:SetActive(true)
	end
end

local function UpdateRoleType(type)
	type = type or Inspect.Unit.Detail(player).role
	if currentRole:GetType() ~= type then
		currentRole:SetType(type)
		UpdateMenu()
	end
end

function Roles.GetCurrentRole()
	return currentRole
end

local function SetCurrentRole(slot)
	if not roles[slot] then
		Roles.UpdateList(roles)
		UpdateMenu()
	end
	currentRole = roles[slot]
	UpdateRoleType()
	menu:UpdateButton(currentRole)
end

local function RoleSlotChange(_, slot)
	if player and currentRole ~= roles[slot] then
		SetCurrentRole(slot)
	end
end
Command.Event.Attach(Event.TEMPORARY.Role, RoleSlotChange, "RoleSlotChange")

local function RoleTypeChange(_, units)
	for id, type in pairs(units) do
		if id == player then
			UpdateRoleType(type)
			menu:UpdateButton(currentRole)
		end
	end
end
Command.Event.Attach(Event.Unit.Detail.Role, RoleTypeChange, "RoleTypeChange")

local function UnitAvailabilityFull(handle, units)
	for id, specifier in pairs(units) do
		if specifier == "player" then
			Command.Event.Detach(Event.Unit.Availability.Full, UnitAvailabilityFull)
			player = id
			SetCurrentRole(Inspect.TEMPORARY.Role())
			print("Roles loaded. Type /roles to open configuration window.")
		end
	end
end
Command.Event.Attach(Event.Unit.Availability.Full, UnitAvailabilityFull, "UnitAvailabilityFull")

local function AddonSavedVariablesLoadEnd(handle, identifier)
	if identifier == addon.identifier then
		Command.Event.Detach(Event.Addon.SavedVariables.Load.End, AddonSavedVariablesLoadEnd)
		_roles = _roles or {}
		_config = _config or {}
		local configDefault =
		{
			sort = "Slot",
			lock = false,
			width = Roles.defaultWidth,
			extend = "Down",
			location = {}
		}
		Utility.SetDefaultValues(_config, configDefault)
		if not _config.lock then
			menu:EnableDrag(true, "Right")
		end
		_config.width = math.min(math.max(_config.width, Roles.minWidth), Roles.maxWidth)
		menu:SetWidth(_config.width)
		menu:SetLayout(_config.extend, Roles.menuSpacing)
		menu:SaveLocation(_config.location)
		if _config.location.x and _config.location.y then
			menu:SetLocation(_config.location.x, _config.location.y)
		else
			menu:Center()
		end
		menu.OnMiddleClick = ShowConfig
		menu:EnableClick(true, "Middle")
	end
end
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, AddonSavedVariablesLoadEnd, "AddonSavedVariablesLoadEnd")

function Roles.SlashCommand(_, option)
	if option == "" then
		ShowConfig()
	elseif option == "toggle" then
		if Inspect.System.Secure() then
			print("Cannot open menu in combat.")
		elseif config:IsOpen() then
			print("Cannot open menu while configuration window is open.")
		else
			menu:Toggle()
		end
	elseif option == "lock" then
		menu:EnableDrag(_config.lock, "Right")
		_config.lock = not _config.lock
		Roles.UpdateLockButton()
		if _config.lock then
			print("Menu location has been locked.")
		else
			print("Menu location has been unlocked.")
		end
	elseif option == "reset" then
		menu:Center()
		print("Menu location has been reset to center.")
	else
		print("Command options:")
		print("/roles (open configuration window)")
		print("/roles toggle (open/close menu)")
		print("/roles lock (lock/unlock menu location)")
		print("/roles reset (reset menu location to center)")
	end
end
Command.Event.Attach(Command.Slash.Register("roles"), Roles.SlashCommand, "RolesSlashCommand")
